#include "libcell.h"

/*  Note:

An alias can only reference to tags either within it's own program or Controller Scoped tags.
An alias can not reference to tags within other programs.  This makes alias checking 
quite a bit easier as we only need to check two places...   

This routine will search for alias references.  If one is found, the alias tag ID data will be
rewrote to point to the actual data object.  The rest of the data remains untouched.  Hopefully
this will work correctly....

*/



void aliascheck (_tag_detail *tag, _tag_data *proglist, _tag_data *mainlist, int debug)
{
int y;

if (tag == NULL)
	return;
if (tag->type >> 8 == 0x10)
	return;


	

if (tag->id == 0)	/* Is the tag Controller Scoped? */
	{
	if (tag->topbase == tag->base) /* if these two are equal, this isn't an alias */
		return;
		
	if (tag->topbase != tag->base)
		{
		for (y=0; y<mainlist->count; y++)
			{
			if (mainlist->tag[y]->base == tag->topbase)
				{
				dprint (DEBUG_VALUES,"%s aliases to %s - %08lX\n",tag->name, mainlist->tag[y]->name, mainlist->tag[y]->base);
				tag->alias_topbase = mainlist->tag[y]->base;
				tag->alias_base = mainlist->tag[y]->base;
				return;
				}
			}
		}
	printf ("WARN: Could not find alias tag %s - %08lX\n",tag->name, tag->topbase);
	return;
	}

if (proglist == NULL)
	return;
	
if (tag->id != 0)	/* We are checking a program scoped tag here against the other program tags */
	{
	if (tag->id == tag->topbase) /* If equal, we aren't an alias. */
		return;
	dprint (DEBUG_VALUES, "Searching for alias to tag %s    ",tag->name);		
	if (tag->id != tag->topbase)
		{
		for (y=0; y < proglist->count; y++)
			{
			if (proglist->tag[y]->id == tag->topbase)
				{
				dprint (DEBUG_VALUES, "%s aliases to %s - %08lX\n",tag->name, proglist->tag[y]->name, proglist->tag[y]->id);
				tag->alias_topbase = proglist->tag[y]->topbase;
				tag->alias_id = proglist->tag[y]->id;
				return;
				}
			}
	/* We didn't find a match in the program scoped tags, search controller scoped tags now... */
		dprint (DEBUG_VALUES, "Searching Controller tags...   ");			
		for (y=0; y < mainlist->count; y++)
			{
			if (mainlist->tag[y]->base == tag->topbase)
				{
				dprint (DEBUG_VALUES, "%s aliases to controller scoped tag %s - %08lX\n",tag->name, mainlist->tag[y]->name, mainlist->tag[y]->base);
				tag->alias_topbase = mainlist->tag[y]->topbase;
				tag->alias_base = mainlist->tag[y]->base;
				tag->alias_id = mainlist->tag[y]->id;
				return;
				}
			}
		dprint (DEBUG_VALUES, "Nothing found.\n");
		return;	
		}
	}
return;	/* If we get here, we have problems...  */
}	