#include "libcell.h"

void get_backplane_data (_comm_header *comm, _backplane_data *backplane, _rack *rack, int debug)
{
int x;
_data_buffer *buff;
_encaps_header *head;
_data_buffer *cpf_buffer;
_data_buffer *sendbuff;
_ioi_data *ioi1;

dprint (DEBUG_TRACE,"get_backplane_data.c entered.\n");
if (backplane == NULL)
	{
	printf ("Backplane structure not allocated - get_backplane_data.c\n");
	return;
	}

if (rack == NULL)
	{
	printf ("Rack structure not allocated - get_backplane_data.c\n");
	return;
	}
	
buff = malloc(DATA_Buffer_Length);
if (buff == NULL)
	{
	printf ("Could not allocate memory for data buffer - get_backplane_data.c\n");
	return;
	}
head = malloc(ENCAPS_Header_Length);
if (head == NULL)
	{
	printf ("Could not allocate memory for Encaps Header - get_backplane_data.c\n");
	free (buff);
	return;
	}
cpf_buffer = malloc(DATA_Buffer_Length);
if (cpf_buffer == NULL)
	{
	printf ("Could not allocate memory for cpf_buffer - get_backplane_data.c\n");
	free(buff);
	free(head);
	return;
	}
sendbuff = malloc(DATA_Buffer_Length);
if (sendbuff == NULL)
	{
	printf ("Could not allocate memory for send buffer - get_backplane_data.c\n");
	free(buff);
	free(head);
	free(cpf_buffer);
	return;
	}
ioi1 = malloc(sizeof(_ioi_data));
if (ioi1 == NULL)
	{
	printf ("Could not allocate memory for ioi structure - get_backplane_data.c\n");
	free (buff);
	free (head);
	free (cpf_buffer);
	free (sendbuff);
	return;
	}

memset(buff,0,DATA_Buffer_Length);
memset(head,0,ENCAPS_Header_Length);
memset(cpf_buffer,0,DATA_Buffer_Length);
memset(sendbuff,0,DATA_Buffer_Length);
memset(ioi1,0, sizeof(_ioi_data));
cpf_buffer->data[cpf_buffer->len++] = GET_ATTRIBUTE_ALL;
ioi1->class = BACKPLANE_DATA;
ioi1->instance = FIRST_INSTANCE;
ioi1->member = -1;
ioi1->point = -1;
ioi1->attribute = -1;
ioi1->tagname = NULL;

ioi(cpf_buffer, ioi1, NULL,debug);

sendRRdata (10, comm, head, buff, debug);
cpf (CPH_Null, NULL, CPH_Unconnected_message, cpf_buffer, buff, debug);

head->len = buff->len;
memcpy (sendbuff->data, head, ENCAPS_Header_Length);
sendbuff->overall_len = ENCAPS_Header_Length;
if (debug!=DEBUG_NIL)
	{
	dprint (DEBUG_BUILD,"Loading sendbuffer with command data.\n");
	for (x=0;x<buff->len;x++)
		dprint (DEBUG_BUILD,"%02X ",buff->data[x]);
	dprint (DEBUG_BUILD,"\n");
	}
memcpy (sendbuff->data + sendbuff->overall_len, buff->data, buff->len);
sendbuff->overall_len += buff->len;
senddata (sendbuff, comm, debug);
readdata (buff, comm, debug);
memset (head,0, ENCAPS_Header_Length);
memcpy (head,buff->data,ENCAPS_Header_Length);
if (head->status != 0)
	{
	dprint (DEBUG_VALUES,"Get Backplane Data command returned an error.\n");
	free (buff);
	free (head);
	free (cpf_buffer);
	free (sendbuff);
	free (ioi1);
	return;
	}
//buff = buff + ENCAPS_Header_Length;
memset (backplane,0, sizeof(_backplane_data));
memcpy (backplane, buff->data+44, sizeof(_backplane_data));
rack->slot = backplane->address;
rack->size = backplane->rack_size;
if (debug != DEBUG_NIL)
	{
	dprint (DEBUG_VALUES,"Got good reply to Get Backplane Data Command\n");	
	for (x=44;x<buff->overall_len;x++)
		dprint (DEBUG_VALUES,"%02X ",buff->data[x]);
	dprint (DEBUG_VALUES,"\n");
	}


free (buff);
free (head);
free (cpf_buffer);
free (sendbuff);
free (ioi1);
dprint (DEBUG_TRACE,"Exiting get_backplane_data.c\n");
return;
}
